
# q-expansion of newform 5184.2.a.r, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5184
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [2], [3], [-2], [-3], [1], [-6], [-6], [-4], [4], [9], [1], [-6], [-12], [-3], [-8], [-5], [-12], [11], [-4], [-12], [6], [5], [0], [14], [-3], [16], [6], [2], [0], [-3], [19], [6], [-10], [4], [4], [-12], [6], [-12], [-14], [-18], [5], [12], [-10], [-20], [26], [-21], [-14], [3], [6], [-7], [-21], [-21], [18], [24], [20], [10], [6], [4], [-30], [7], [-18], [29], [18], [4], [-1], [-33], [16], [-21], [-18], [-28], [34], [-23], [0], [-18], [-20], [-27], [17], [12], [-20], [-30], [-7], [8], [-3], [9], [17], [30], [20], [15], [-42], [26], [15], [13], [0], [0], [-3], [-20], [4], [1], [30], [39], [45], [37], [11], [9], [6], [12], [-37], [-28], [16], [27], [-35], [-40], [-3], [-23], [18], [6], [36], [4], [-22], [-36], [9], [-8], [-30], [4], [-36], [26], [-14], [-47], [6], [8], [-2], [42], [2], [-18], [4], [12], [33], [7], [18], [-16], [12], [10], [12], [-26], [-42], [-35], [24], [-8], [-42], [19], [0], [31], [48], [38], [-6], [14], [60], [-3], [9], [-22], [36], [-51], [0], [-16], [28], [2], [0], [3], [40], [36], [11], [38], [30], [1], [-54], [-28], [-8], [38], [9], [22], [27], [54], [0], [16], [-56], [29], [60], [-1], [15], [-59], [-36], [-33], [3], [47], [4], [-9], [6], [30], [-58], [-56], [35], [-21], [66], [-64], [27], [27], [-17], [-67], [-30], [62], [-36], [24], [-55], [68], [21], [66], [-48], [34], [-40], [-27], [38], [-51], [22], [-6], [12], [-34], [-24], [-26], [52], [-22], [-9], [16], [42], [-70], [18], [57], [-42], [45], [-20], [-4], [-38], [-33], [-48], [-28], [-45], [-68], [-54], [22], [27], [12], [26], [42], [39], [16], [28], [0], [-43], [32], [15], [4], [-14], [-18], [55], [12], [75], [-41], [30], [28], [-68], [35], [50], [23], [20], [-36], [10], [-25], [27], [-18], [-58], [-30], [-26], [49], [-18], [26], [-48], [-28], [63], [24], [-39], [-69], [-51], [64], [-66], [50], [-54], [36], [-68], [-7], [18], [-16], [36], [-8], [2], [-60], [40], [-6], [-80], [-48], [72], [66], [16], [-12], [11], [-33], [12], [-37], [6], [-35], [29], [-84], [2], [18], [41], [25], [-71], [-60], [-30], [-80], [-12], [-16], [-21], [-47], [-84], [-2], [63], [23], [20], [-32], [6], [12], [20], [18], [33], [52], [-17], [-72], [78], [28], [89], [78], [-28], [76], [-42], [-84], [-60], [-6], [6], [22], [15], [-72], [-45], [-53], [-25], [72], [26], [-19], [-60], [85], [36], [36], [38], [46], [-87], [-54], [-10], [69], [26], [-42], [-15], [-40], [-45], [-65], [60], [-10], [-80], [-80], [-12], [-1], [-36], [-33], [97], [66], [-58], [32], [-51], [-41], [72], [-14], [-39], [-76], [42], [-18], [20], [94], [69], [-59], [-36], [71], [102], [33], [-17], [62], [-6], [66], [14], [21], [0], [-60], [-20], [-36], [-51], [-70], [60], [39], [-54], [52], [0]]
